from contextvars import copy_context
from unittest import TestCase

from fred_types.translations import LANGUAGE, LANGUAGE_GETTER, gettext_lazy


class GettextLazyTest(TestCase):
    def setUp(self):
        self.context = copy_context()

    def test_gettext_eq(self):
        data = (
            (gettext_lazy("Kryten"), "Kryten", True),
            (gettext_lazy("Kryten"), gettext_lazy("Kryten"), True),
            (gettext_lazy("Kryten"), "Kryt", False),
            (gettext_lazy("Kryten"), gettext_lazy("Kryt"), False),
        )
        for first, second, result in data:
            with self.subTest(first=first, second=second):
                self.assertEqual(first == second, result)
                self.assertEqual(second == first, result)

    def test_gettext_ne(self):
        data = (
            (gettext_lazy("Kryten"), "Kryten", False),
            (gettext_lazy("Kryten"), gettext_lazy("Kryten"), False),
            (gettext_lazy("Kryten"), "Kryt", True),
            (gettext_lazy("Kryten"), gettext_lazy("Kryt"), True),
        )
        for first, second, result in data:
            with self.subTest(first=first, second=second):
                self.assertEqual(first != second, result)
                self.assertEqual(second != first, result)

    def test_gettext_repr(self):
        self.assertEqual(repr(gettext_lazy("Reserved")), "<_LazyTranslation: 'Reserved'>")

    def test_gettext(self):
        text = gettext_lazy("Reserved")
        self.assertEqual(text, "Reserved")

    def test_gettext_translated(self):
        def do_test():
            text = gettext_lazy("Reserved")
            LANGUAGE.set("cs")
            self.assertEqual(text, "Rezervováno")

        self.context.run(do_test)

    def test_language_getter(self):
        # Test active language is determined by the LANGUAGE_GETTER if used.
        def get_language():
            return "cs"

        def do_test():
            text = gettext_lazy("Reserved")
            LANGUAGE_GETTER.set(get_language)
            self.assertEqual(text, "Rezervováno")

        self.context.run(do_test)
